package com.fs.company.controller.qw;


import com.fs.common.annotation.Log;
import com.fs.common.core.controller.BaseController;
import com.fs.common.core.domain.AjaxResult;
import com.fs.common.core.domain.R;
import com.fs.common.core.page.TableDataInfo;
import com.fs.common.enums.BusinessType;
import com.fs.framework.service.TokenService;
import com.fs.qw.mapper.QwUserMapper;
import com.fs.qw.param.SopUserLogsVO;
import com.fs.qw.service.IQwUserService;
import com.fs.qw.vo.AddSopUserGroupChat;
import com.fs.qw.vo.UpdateSopUserLogDateVo;
import com.fs.sop.domain.QwSop;
import com.fs.sop.domain.SopUserLogs;
import com.fs.sop.mapper.QwSopMapper;
import com.fs.sop.params.SopUserLogsParam;
import com.fs.sop.service.ISopUserLogsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

/**
 * sopUserLogsController
 *
 * @author fs
 * @date 2024-12-27
 */
@RestController
@RequestMapping("/qwSop/sopUserLogs")
public class SopUserLogsController extends BaseController
{
    @Autowired
    private ISopUserLogsService sopUserLogsService;
    @Autowired
    private TokenService tokenService;

    @Autowired
    private QwUserMapper qwUserMapper;

    @Autowired
    private QwSopMapper sopMapper;

    /**
     * 查询sopUserLogs列表
     */
    @PreAuthorize("@ss.hasPermi('qw:sop:list')")
    @GetMapping("/list")
    public TableDataInfo list(SopUserLogsParam sopUserLogs)
    {
        sopUserLogs.setStatus(1);

        QwSop qwSop = sopMapper.selectQwSopById(sopUserLogs.getSopId());
        Integer filterMode = qwSop.getFilterMode();

        List<SopUserLogsVO> list=null;
        if(filterMode == null || filterMode == 1){
            startPage();
             list = sopUserLogsService.selectSopUserLogsList(sopUserLogs);
        }else {
            startPage();
            list = sopUserLogsService.selectSopUserLogsGroupListByParam(sopUserLogs);
        }

        list.forEach(item->{
            item.setQwUserName(qwUserMapper.selectQwUserByQwUserIdAndCorpId(item.getQwUserId(), item.getCorpId()));
        });

        return getDataTable(list);
    }

//    /**
//     * 导出sopUserLogs列表
//     */
//    @PreAuthorize("@ss.hasPermi('qwSop:sopUserLogs:export')")
//    @Log(title = "sopUserLogs", businessType = BusinessType.EXPORT)
//    @GetMapping("/export")
//    public AjaxResult export(SopUserLogs sopUserLogs)
//    {
//        List<SopUserLogs> list = sopUserLogsService.selectSopUserLogsList(sopUserLogs);
//        ExcelUtil<SopUserLogs> util = new ExcelUtil<SopUserLogs>(SopUserLogs.class);
//        return util.exportExcel(list, "sopUserLogs数据");
//    }

    /**
     * 获取sopUserLogs详细信息
     */
    @PreAuthorize("@ss.hasPermi('qw:sop:list')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id)
    {
        return AjaxResult.success(sopUserLogsService.selectSopUserLogsById(id));
    }

    /**
    * 修复营期
    */

    @Log(title = "repairSopUserLogs", businessType = BusinessType.UPDATE)
    @PostMapping("/repairSopUserLogs")
    public R repairSopUserLogs(@RequestBody SopUserLogs sopUserLogs){

        return sopUserLogsService.repairSopUserLogs(sopUserLogs);
    }


    /**
     * 删除sopUserLogs
     */
    @PreAuthorize("@ss.hasPermi('qwSop:sopUserLogs:remove')")
    @Log(title = "删除sopUserLogs", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable String[] ids)
    {
        return toAjax(sopUserLogsService.deleteSopUserLogsByIds(ids));
    }


    @Log(title = "修改群营期时间", businessType = BusinessType.UPDATE)
    @PostMapping("/updateLogDate")
    public R updateLogDate(@RequestBody UpdateSopUserLogDateVo vo){
        sopUserLogsService.updateLogDate(vo);
        return R.ok();
    }
    @Log(title = "追加SOP群聊", businessType = BusinessType.UPDATE)
    @PostMapping("/addGroupChat")
    public R addGroupChat(@RequestBody AddSopUserGroupChat vo){
        sopUserLogsService.addGroupChat(vo);
        return R.ok();
    }
}
