package com.fs.user;

import com.alibaba.fastjson.JSON;
import com.fs.common.core.controller.BaseController;
import com.fs.common.core.domain.R;
import com.fs.common.core.page.TableDataInfo;
import com.fs.common.utils.ServletUtils;
import com.fs.company.cache.ICompanyUserCacheService;
import com.fs.framework.security.LoginUser;
import com.fs.framework.service.TokenService;

import com.fs.his.service.IFsUserService;
import com.fs.qw.domain.CustomerTransferApproval;
import com.fs.qw.dto.FsUserTransferParamDTO;
import com.fs.qw.service.ICustomerTransferApprovalService;
import com.fs.store.param.h5.FsUserPageListParam;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;

@Api(tags = "会员管理接口")
@RestController
@Slf4j
@RequestMapping("/user/fsUser")
@AllArgsConstructor
public class FsUserAdminController extends BaseController {

    @Autowired
    private IFsUserService fsUserService;

    @Autowired
    private ICompanyUserCacheService companyUserCacheService;

    @Autowired
    private TokenService tokenService;

    @Autowired
    private ICustomerTransferApprovalService transferApprovalService;

    @PreAuthorize("@ss.hasPermi('user:fsUser:list')")
    @PostMapping("/list")
    @ApiOperation("会员列表(与移动端使用的相同查询)")
    public TableDataInfo pageList(@RequestBody FsUserPageListParam param) {
        //startPage();

        LoginUser loginUser = tokenService.getLoginUser(ServletUtils.getRequest());
        if (param.isMyFsUser()){
            param.setCompanyId(loginUser.getCompany().getCompanyId());
            param.setCompanyUserId(String.valueOf(loginUser.getUser().getUserId()));
        }else {
            param.setCompanyId(loginUser.getCompany().getCompanyId());
        }
        if(param.getCompanyUserId() == null) {
            throw new IllegalArgumentException("当前销售不存在！");
        }
        return fsUserService.selectFsUserPageListNew(param);
    }

    @PostMapping("/auditUser")
    @ApiOperation("审核用户（移除小黑屋）")
    public R auditUser(@RequestBody String[] userIds) {
        Boolean r = fsUserService.disabledUser(userIds, true);
        if (r) {
            return R.ok();
        }
        return R.error();
    }

    /**
     * 转移
     * @param param
     */
    @PostMapping("/transfer")
    public R transfer(@RequestBody FsUserTransferParamDTO param){
        log.info("客户转移: {}",param);
        LoginUser loginUser = tokenService.getLoginUser(ServletUtils.getRequest());
        param.setSourceCompanyUserId(loginUser.getUser().getUserId());
        CustomerTransferApproval customerTransferApproval = new CustomerTransferApproval();
        customerTransferApproval.setCorpId(String.valueOf(loginUser.getCompany().getCompanyId()));
        customerTransferApproval.setCustomerIds(JSON.toJSONString(param.getUserIds()));
        customerTransferApproval.setOriginalUserId(param.getSourceCompanyUserId());
        customerTransferApproval.setTargetUserId(param.getTargetCompanyUserId());
        customerTransferApproval.setInitiatorUserId(param.getSourceCompanyUserId());
        customerTransferApproval.setContent(param.getContent());
        customerTransferApproval.setCreatedAt(new Date());
        customerTransferApproval.setUpdatedAt(new Date());
        customerTransferApproval.setApprovalStatus(0);
        customerTransferApproval.setTransferType(2);
        transferApprovalService.insertCustomerTransferApproval(customerTransferApproval);
        return R.ok("转移申请已经提交，等待转移成功!");
    }


}
