package com.fs.his.service.impl;

import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.math.MathContext;
import java.math.RoundingMode;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.fs.common.constant.HttpStatus;
import com.fs.common.core.domain.R;
import com.fs.common.core.domain.ResponseResult;
import com.fs.common.core.page.TableDataInfo;
import com.fs.common.utils.DateUtils;
import com.fs.common.utils.DictUtils;
import com.fs.common.utils.ParseUtils;
import com.fs.common.utils.StringUtils;
import com.fs.company.cache.ICompanyTagCacheService;
import com.fs.company.cache.ICompanyUserCacheService;
import com.fs.company.domain.*;
import com.fs.company.mapper.*;
import com.fs.company.service.ICompanyTagService;
import com.fs.course.domain.FsUserCompanyUser;
import com.fs.course.mapper.FsUserCompanyUserMapper;
import com.fs.course.mapper.FsUserCourseVideoMapper;
import com.fs.course.param.CourseAnalysisParam;
import com.fs.course.param.newfs.FsUserCourseBeMemberParam;
import com.fs.course.param.newfs.UserCourseVideoPageParam;
import com.fs.course.vo.newfs.FsCourseAnalysisCountVO;
import com.fs.course.vo.newfs.FsCourseAnalysisVO;
import com.fs.course.vo.newfs.FsUserCourseVideoPageListVO;
import com.fs.his.config.IntegralConfig;
import com.fs.his.domain.FsUserAddress;
import com.fs.his.domain.FsUserIntegralLogs;
import com.fs.his.mapper.*;
import com.fs.his.param.FsUserParam;
import com.fs.his.vo.FsUserExportListVO;
import com.fs.his.vo.FsUserFollowDoctorVO;
import com.fs.his.vo.FsUserVO;
import com.fs.his.vo.UserVo;
import com.fs.qw.cache.IQwExternalContactCacheService;
import com.fs.qw.dto.FsUserTransferParamDTO;
import com.fs.qw.param.QwFsUserParam;
import com.fs.qw.vo.QwFsUserVO;
import com.fs.store.domain.FsUserCourseCount;
import com.fs.store.param.h5.FsUserPageListParam;
import com.fs.store.param.h5.UserStatisticsCommonParam;
import com.fs.store.service.cache.IFsUserCourseCountCacheService;
import com.fs.store.vo.h5.*;
import com.fs.system.service.ISysConfigService;
import com.fs.watch.domain.WatchUser;
import com.fs.watch.service.WatchUserService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.http.util.Asserts;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.fs.his.domain.FsUser;
import com.fs.his.service.IFsUserService;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import static com.fs.his.utils.PhoneUtil.encryptPhone;

/**
 * 用户Service业务层处理
 *
 * @author fs
 * @date 2023-06-07
 */
@Service
public class FsUserServiceImpl implements IFsUserService
{
    Logger logger= LoggerFactory.getLogger(getClass());;
    @Autowired
    private FsUserMapper fsUserMapper;
    @Autowired
    private CompanyUserMapper companyUserMapper;
    @Autowired
    private CompanyMapper companyMapper;
    @Autowired
    private FsUserAddressMapper fsUserAddressMapper;
    @Autowired
    private FsUserCourseVideoMapper userCourseVideoMapper;
    @Autowired
    private FsStoreOrderMapper fsStoreOrderMapper;
    @Autowired
    private FsFollowMapper fsFollowMapper;
    @Autowired
    private WatchUserService watchUserService;

    @Autowired
    private ICompanyTagService companyTagService;

    @Autowired
    CompanyTagMapper companyTagMapper;

    @Autowired
    private CompanyTagUserMapper companyTagUserMapper;
    @Autowired
    private FsUserCompanyUserMapper fsUserCompanyUserMapper;

    @Autowired
    private ICompanyUserCacheService companyUserCacheService;

    @Autowired
    private IFsUserCourseCountCacheService fsUserCourseCountCacheService;

    @Autowired
    private ICompanyTagCacheService companyTagCacheService;

    @Autowired
    private IQwExternalContactCacheService qwExternalContactCacheService;

    @Autowired
    private CompanyRoleMapper companyRoleMapper;


    /**
     * 查询用户
     *
     * @param userId 用户主键
     * @return 用户
     */
    @Override
    public FsUser selectFsUserByUserId(Long userId)
    {
        return fsUserMapper.selectFsUserByUserId(userId);
    }

    /**
     * 查询用户列表
     *
     * @param fsUser 用户
     * @return 用户
     */
    @Override
    public List<FsUser> selectFsUserList(FsUser fsUser)
    {
        return fsUserMapper.selectFsUserList(fsUser);
    }

    /**
     * 新增用户
     *
     * @param fsUser 用户
     * @return 结果
     */
    @Override
    @Transactional
    public int insertFsUser(FsUser fsUser)
    {
        fsUser.setCreateTime(DateUtils.getNowDate());
        if (StringUtils.isNotEmpty(fsUser.getPhone())){
            fsUser.setPhone(encryptPhone(fsUser.getPhone()));
        }
        int i = fsUserMapper.insertFsUser(fsUser);

        //生成腕表模块默认设置
        WatchUser watchUser = new WatchUser();
        watchUser.setUserId(fsUser.getUserId());
        watchUserService.insert(watchUser);
        return i;
    }

    /**
     * 修改用户
     *
     * @param fsUser 用户
     * @return 结果
     */
    @Override
    public int updateFsUser(FsUser fsUser)
    {
        fsUser.setUpdateTime(DateUtils.getNowDate());
        if (fsUser.getPhone()!=null&&fsUser.getPhone().length()==11&&fsUser.getPhone().matches("\\d+")){
            fsUser.setPhone(encryptPhone(fsUser.getPhone()));
        }else {
            fsUser.setPhone(null);
        }
        return fsUserMapper.updateFsUser(fsUser);
    }

    /**
     * 批量删除用户
     *
     * @param userIds 需要删除的用户主键
     * @return 结果
     */
    @Override
    public int deleteFsUserByUserIds(Long[] userIds)
    {
        for (Long userId : userIds) {
            fsUserMapper.updateFsUserByUserId(userId);
        }
        return userIds.length;
    }

    /**
     * 删除用户信息
     *
     * @param userId 用户主键
     * @return 结果
     */
    @Override
    public int deleteFsUserByUserId(Long userId)
    {
        return fsUserMapper.updateFsUserByUserId(userId);
    }

    @Override
    public List<FsUserVO> selectFsUserListVO(FsUserParam fsUser) {
        return fsUserMapper.selectFsUserListVO(fsUser);
    }

    @Override
    public FsUser selectFsUserByOpenId(String openId) {
        return fsUserMapper.selectFsUserByOpenId(openId);
    }

    @Override
    public FsUser selectFsUserByCourseMaOpenId(String openId) {
        return fsUserMapper.selectFsUserByCourseMaOpenId(openId);
    }

    @Override
    public FsUser selectFsUserByPhone(String phone) {
        return fsUserMapper.selectFsUserByPhone(phone);
    }

    @Override
    public List<FsUserAddress> selectFsUserAddressByUserId(Long userId) {
        FsUserAddress fsUserAddress = new FsUserAddress();
        fsUserAddress.setUserId(userId);
        return fsUserAddressMapper.selectFsUserAddressList(fsUserAddress);
    }

    @Override
    public List<UserVo> selectUserVOList(String name) {
        return fsUserAddressMapper.selectUserVOList(name);
    }

    @Override
    public List<FsUserVO> selectFsUserListVOByComponentsUser(FsUserParam fsUser) {
        return fsUserMapper.selectFsUserListVOByComponentsUser(fsUser);
    }

    @Override
    public List<FsUserExportListVO> selectFsUserExportListVO(FsUserParam fsUser) {
        return fsUserMapper.selectFsUserExportListVO(fsUser);
    }
    @Autowired
    private FsUserIntegralLogsMapper integralLogsMapper;

    @Autowired
    private ISysConfigService configService;

    @Override
    @Transactional
    public void addUserIntegral(BigDecimal payMoney,Long userId,Long orderId ,Integer orderType) {
            FsUser fsUser = fsUserMapper.selectFsUserByUserId(userId);
            if(fsUser!=null){
                String json =configService.selectConfigByKey("his.integral");
                IntegralConfig config= JSONUtil.toBean(json,IntegralConfig.class);
                Long integral = payMoney.divide(BigDecimal.valueOf(config.getIntegralRatio()),new MathContext(10, RoundingMode.HALF_UP)).longValue();
                //实付多少获取多少积分
                if (integral>0){
                    FsUser u = new FsUser();
                    u.setUserId(userId);
                    u.setIntegral(fsUser.getIntegral()+integral);
                    fsUserMapper.updateFsUser(u);
                    logger.info("用户{}获取积分{}",fsUser.getUserId(),integral);
                    //写入日志
                    FsUserIntegralLogs logs = new FsUserIntegralLogs();
                    logs.setIntegral(integral.longValue());
                    logs.setBusinessType(orderType);
                    logs.setUserId(userId);
                    logs.setBalance(u.getIntegral());
                    logs.setLogType(2);
                    logs.setBusinessId(orderId.toString());
                    logs.setCreateTime(new Date());
                    integralLogsMapper.insertFsUserIntegralLogs(logs);
                }
            }
    }

    @Override
    @Transactional
    public void followAddIntegral(Long userId,Long orderId) {
        FsUser fsUser = fsUserMapper.selectFsUserByUserId(userId);
        if(fsUser!=null){
            String json =configService.selectConfigByKey("his.integral");
            IntegralConfig config= JSONUtil.toBean(json,IntegralConfig.class);
            Integer todayTotalIntegral= integralLogsMapper.selectTodayTotalIntegral(config.getIntegralTypeByOneDay(),userId);
            if (todayTotalIntegral==null){
                todayTotalIntegral=0;
            }
            if (todayTotalIntegral!=null&&todayTotalIntegral>=config.getIntegralByOneDay()){
                return;
            }
            Long integral = config.getIntegralFollow()+0L;
                FsUser u = new FsUser();
                u.setUserId(userId);
                u.setIntegral(fsUser.getIntegral()+integral);
                fsUserMapper.updateFsUser(u);
                logger.info("用户{}获取积分{}",fsUser.getUserId(),integral);
                //写入日志
                FsUserIntegralLogs logs = new FsUserIntegralLogs();
                logs.setIntegral(integral.longValue());
                logs.setBusinessType(4);
                logs.setUserId(userId);
                logs.setBalance(u.getIntegral());
                logs.setLogType(6);
                logs.setBusinessId(orderId.toString());
                logs.setCreateTime(new Date());
                integralLogsMapper.insertFsUserIntegralLogs(logs);

        }
    }

    @Transactional
    public void subIntegral(Long userId) {
        FsUser fsUser = fsUserMapper.selectFsUserByUserId(userId);
        if(fsUser!=null&&fsUser.getIntegral()!=0){
            Long integral = fsUser.getIntegral();
            fsUser.setIntegral(0L);
            fsUserMapper.updateFsUser(fsUser);
            logger.info("用户{}扣除积分{}",fsUser.getUserId(),0-integral);
            FsUserIntegralLogs logs = new FsUserIntegralLogs();
            logs.setIntegral(0-integral);
            logs.setUserId(userId);
            logs.setBalance(0L);
            logs.setLogType(7);
            logs.setCreateTime(new Date());
            integralLogsMapper.insertFsUserIntegralLogs(logs);
            }

        }

    @Override
    public FsUserFollowDoctorVO getUserFollowDoctorAndOrderId(Long userId) {
        FsUserFollowDoctorVO fsUserFollowDoctorVO = fsStoreOrderMapper.selectFsFollowDoctorIdByUserId(userId);
        if (fsUserFollowDoctorVO!=null&&fsUserFollowDoctorVO.getFollowDoctorId()!=null){
            Long followId = fsFollowMapper.selectFsFollowByStoreOrderId(fsUserFollowDoctorVO.getOrderId());
            fsUserFollowDoctorVO.setFollowId(followId);
        }
        return fsUserFollowDoctorVO;
    }

    @Override
    public List<FsUserVO> selectFsUserListVOByComponent(FsUserParam fsUser) {
        return fsUserMapper.selectFsUserListVOByComponent(fsUser);
    }


    @Override
    @Transactional
    public void subUserIntegral(Long userId,Long orderId ,Integer orderType) {
            FsUser fsUser = fsUserMapper.selectFsUserByUserId(userId);
            if(fsUser!=null){
                FsUserIntegralLogs maps=new FsUserIntegralLogs();
                maps.setLogType(2);
                maps.setBusinessType(orderType);
                maps.setBusinessId(orderId.toString());
                List<FsUserIntegralLogs> logsList=integralLogsMapper.selectFsUserIntegralLogsList(maps);
                if(logsList.size()>0){
                    for (FsUserIntegralLogs userLogs : logsList) {
                        fsUser.setIntegral(fsUser.getIntegral()-userLogs.getIntegral());
                        fsUserMapper.updateFsUser(fsUser);
                        logger.info("用户{}扣除积分{}",fsUser.getUserId(),userLogs.getIntegral());
                        //写入日志
                        FsUserIntegralLogs logs = new FsUserIntegralLogs();
                        logs.setIntegral(-logsList.get(0).getIntegral());
                        logs.setUserId(userId);
                        logs.setBalance(fsUser.getIntegral());
                        logs.setBusinessType(orderType);
                        logs.setLogType(4);
                        logs.setBusinessId(orderId.toString());
                        logs.setCreateTime(new Date());
                        integralLogsMapper.insertFsUserIntegralLogs(logs);
                    }
                }

            }

    }

    @Override
    public void shareAddIntegral(Long userId) {
        FsUser fsUser = fsUserMapper.selectFsUserByUserId(userId);
        if(fsUser!=null) {
            String json = configService.selectConfigByKey("his.integral");
            IntegralConfig config = JSONUtil.toBean(json, IntegralConfig.class);
            Integer integral = config.getIntegralRatio();
            FsUser u = new FsUser();
            u.setUserId(userId);
            u.setIntegral(fsUser.getIntegral() + integral);
            fsUserMapper.updateFsUser(u);
            logger.info("用户{}获取积分{}", fsUser.getUserId(), integral);
            FsUserIntegralLogs logs = new FsUserIntegralLogs();
            logs.setIntegral(integral + 0L);
            logs.setUserId(userId);
            logs.setBalance(fsUser.getIntegral());
            logs.setLogType(3);
            logs.setCreateTime(new Date());
            integralLogsMapper.insertFsUserIntegralLogs(logs);
        }
    }

    @Override
    public FsUser selectFsUserByUnionid(String unionid) {
        return fsUserMapper.selectFsUserByUnionid(unionid);
    }

    @Override
    public List<QwFsUserVO> selectQwFsUserListVO(QwFsUserParam qwFsUserParam) {
        return fsUserMapper.selectQwFsUserListVO(qwFsUserParam);
    }


    @Override
    public R clearUserVip(Long userId) {
        FsUser user = new FsUser();
        user.setUserId(userId);
        user.setIsVip(0);
        user.setVipStartDate(null);
        user.setVipEndDate(null);
        if (fsUserMapper.updateFsUser(user)>0){
            return R.ok("清除成功");
        }
        return R.error("清除vip失败");
    }

    @Override
    public List<QwFsUserVO> selecQwFsUserListVO(QwFsUserParam qwFsUserParam) {
        return fsUserMapper.selecQwFsUserListVO(qwFsUserParam);
    }

    @Override
    public Boolean isEntityNull(FsUserParam param) {
        if (param == null) {
            return true; // 整个实体对象为null
        }
        Class<? extends FsUserParam> entityClass = param.getClass();
        Field[] fields = entityClass.getDeclaredFields();
        try {
            for (Field field : fields) {
                field.setAccessible(true);
                Object value = field.get(param);
                if (value != null) {
                    return false; // 实体对象中有字段不为null
                }
            }
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        }
        return true; // 实体对象内所有字段为null
    }

    @Override
    public Long selectFsUserExportListVOCount(FsUserParam fsUser) {
        return fsUserMapper.selectFsUserExportListVOCount(fsUser);
    }

    @Override
    public FsUser selectFsUserByMpOpenId(String openId) {
        return fsUserMapper.selectFsUserByMpOpenId(openId);
    }

    @Transactional(rollbackFor = Exception.class)
    public void setRepeatFansTag(FsUserCourseBeMemberParam param) {
        Map<String, Object> map = new HashMap<>();
        map.put("tagName", "重粉");
        List<CompanyTag> companyTags = companyTagMapper.selectCompanyTagListByMap(map);
        Long tagId;
        if(companyTags == null || companyTags.isEmpty()){
            CompanyTag companyTag = new CompanyTag();
            companyTag.setCompanyId(param.getCompanyId());
            companyTag.setTag("重粉");
            companyTagService.insertCompanyTag(companyTag);
            tagId = companyTag.getTagId();
        } else {
            tagId = companyTags.get(0).getTagId();
        }
        //  查询
        CompanyTagUser companyTagUserParam = new CompanyTagUser();
        companyTagUserParam.setUserId(param.getUserId());
        companyTagUserParam.setTagIds(tagId.toString());
        List<CompanyTagUser> companyTagUsers = companyTagUserMapper.selectCompanyTagUserList(companyTagUserParam);
        if(companyTagUsers == null || companyTagUsers.isEmpty()){
            CompanyTagUser companyTagUser = new CompanyTagUser();
            companyTagUser.setUserId(param.getUserId());
            companyTagUser.setCompanyId(param.getCompanyId());
            companyTagUser.setCompanyUserId(param.getCompanyUserId());
            companyTagUser.setTagIds(tagId.toString());
            companyTagUser.setCreateTime(new Date());
            companyTagUserMapper.insertCompanyTagUser(companyTagUser);
        }
    }

    @Override
    public PageInfo<FsUserPageListVO> selectFsUserPageList(FsUserPageListParam param) {


//        List<CompanyRole> companyRoles = companyRoleMapper.selectRolePermissionByUserId(param.getUserId());
//        List<CompanyUser> companyUsers;
//        CompanyUser companyUser = companyUserMapper.selectCompanyUserById(param.getUserId());
//        // 判断是否管理员 或者包含 1:全部数据权限
//        if (companyUser.isAdmin() || companyRoles.stream().anyMatch(r -> "1".equals(r.getDataScope()))) {
//            companyUsers = companyUserService.getCompanyUserListByCompanyIdAndDeptId(companyUser.getCompanyId(), null);
//        }
//        // 判断是否包含 3:本部门数据权限 4:本部门及以下数据权限
//        else if (companyRoles.stream().anyMatch(r -> "3".equals(r.getDataScope()) || "4".equals(r.getDataScope()))) {
//            companyUsers = companyUserService.getCompanyUserListByCompanyIdAndDeptId(companyUser.getCompanyId(), companyUser.getDeptId());
//        }
//        // 默认空 -- 判断是否包含 5:仅可查看本人
//        else {
//            companyUsers = new ArrayList<>();
//        }

        if (param.getUserId() != null) {
            CompanyUser companyUser = companyUserMapper.selectCompanyUserById(param.getUserId());
                if (companyUser.isAdmin()){
                    param.setUserId(0L);
                    param.setCompanyId(companyUser.getCompanyId());
                }
        }

        List<FsUserPageListVO> fsUserPageListVOS = fsUserMapper.selectFsUserPageList(param);
        // 获取当前销售所有重粉会员
        List<FsUserCompanyUser> fsUserCompanyUsers = fsUserCompanyUserMapper.selectRepeatUser(param.getUserId());

        if(!fsUserCompanyUsers.isEmpty()){
            List<Long> userIds = fsUserCompanyUsers.stream().map(FsUserCompanyUser::getUserId).collect(Collectors.toList());
            // 获取会员所属的重粉销售
            List<FsUserCompanyUser> repeatCompanyUserNames = fsUserCompanyUserMapper.selectRepeatCompanyUserName(userIds);
            Map<Long, String> map = repeatCompanyUserNames.stream().collect(Collectors.toMap(FsUserCompanyUser::getUserId, FsUserCompanyUser::getRepeatCompanyUserName));

            for (FsUserPageListVO fsUserPageListVO : fsUserPageListVOS) {
                fsUserPageListVO.setRepeatCompanyUserName(map.get(fsUserPageListVO.getUserId()));
                if ("微信用户".equals(fsUserPageListVO.getNickname()) && StringUtils.isNotEmpty(fsUserPageListVO.getPhone())) {
                    fsUserPageListVO.setNickname(fsUserPageListVO.getPhone());
                }
                if(StringUtils.isNotEmpty(fsUserPageListVO.getTag()) && fsUserPageListVO.getIsRepeatFans() == 0){
                    StringBuilder newTag = removeRepeatFansTag(fsUserPageListVO);
                    fsUserPageListVO.setTag(newTag.toString());
                }
            }
            return new PageInfo<>(fsUserPageListVOS);
        } else {
            return new PageInfo<>(fsUserPageListVOS);
        }


    }

    private static StringBuilder removeRepeatFansTag(FsUserPageListVO fsUserPageListVO) {
        String tag = fsUserPageListVO.getTag();
        String[] splitTag = tag.split(",");
        StringBuilder newTag = new StringBuilder();
        for (String s : splitTag) {
            if(!"重粉".equals(s)){
                newTag.append(s);
            }
        }
        return newTag;
    }

    @Override
    public Boolean disabledUser(String[] ids, boolean status) {
        boolean result;
        if (status) {
            // 启用
            result = fsUserMapper.batchUpdateFsUserByIds(ids, 1) > 0;
        } else {
            // 禁用
            result = fsUserMapper.batchUpdateFsUserByIds(ids, 0) > 0;
        }
        return result;
    }

    @Override
    public FsUser selectFsUserById(Long userId) {
        return fsUserMapper.selectFsUserById(userId);
    }

    @Override
    @Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
    public void transfer(FsUserTransferParamDTO param) {
        Asserts.check(ObjectUtils.isNotNull(param.getSourceCompanyUserId()), "来源用户不能为空！");
        Asserts.check(ObjectUtils.isNotNull(param.getTargetCompanyUserId()), "转移用户不能为空！");
        Asserts.check(CollectionUtils.isNotEmpty(param.getUserIds()), "要转移的客户列表不能为空！");

        fsUserMapper.transferCompanyUser(param);
    }

    @Override
    public TableDataInfo selectFsUserPageListNew(FsUserPageListParam param) {
        // 找出下级销售
        String companyUserId = param.getCompanyUserId();
        if(companyUserId != null) {
            Long companyUser = Long.parseLong(companyUserId);
            Set<Long> userIds = companyUserCacheService.selectUserAllCompanyUserId(companyUser);
            param.setCompanyUserIds(userIds);
        }

        List<FsUserPageListVO> fsUserPageListVOS = fsUserMapper.selectFsUserPageListNew(param);
        Map<Long, CompanyTag> tagMap = companyTagCacheService.queryAllTagMap();
        for (FsUserPageListVO item : fsUserPageListVOS) {
            if(item.getCompanyUserId() != null) {
                String companyUserName = companyUserCacheService.selectCompanyUserNameUserById(item.getCompanyUserId());
                if(companyUserName != null) {
                    item.setCompanyUserNickName(companyUserName);
                }
            }
            if(item.getPhone() != null) {
                item.setPhone(ParseUtils.parsePhone(item.getPhone()));
            }
            if(item.getStatus() != null) {
                String userStatus = DictUtils.getDictLabel("user_status", String.valueOf(item.getStatus()));
                if(StringUtils.isNotBlank(userStatus)){
                    item.setStatusText(userStatus);
                }
            }
            if(item.getUserId() != null) {
                FsUserCourseCount byUserId = fsUserCourseCountCacheService.findByUserId(item.getUserId());
                if(byUserId != null) {
                    item.setWatchCourseCount(byUserId.getWatchCourseCount());
                    item.setMissCourseCount(byUserId.getMissCourseCount());
                    item.setMissCourseStatus(byUserId.getMissCourseStatus());
                    if(StringUtils.isNotEmpty(byUserId.getPartCourseCount())){
                        item.setPartCourseCount(Long.valueOf(byUserId.getPartCourseCount()));
                    }
                    item.setCourseCountStatus(byUserId.getStatus());
                    item.setStopWatchDays(byUserId.getStopWatchDays());
                    item.setCompleteWatchDate(byUserId.getCompleteWatchDate());
                }
                String userTagByUserId = companyTagCacheService
                        .findUserTagByUserId(item.getUserId(),item.getCompanyUserId());
                if(StringUtils.isNotEmpty(userTagByUserId)) {
                    String[] split = userTagByUserId.split(",");
                    Set<String> tagNames = new HashSet<>();
                    for (String tag : split) {
                        if(StringUtils.isNotBlank(tag)) {
                            Long tagL = Long.parseLong(tag);
                            CompanyTag companyTag = tagMap.get(tagL);
                            if(companyTag != null) {
                                tagNames.add(companyTag.getTag());
                            }
                        }
                    }
                    item.setTagIds(userTagByUserId);
                    item.setTag(String.join(",",tagNames));
                }

                // 是否宠粉
//                Integer isRepeat = qwExternalContactCacheService.selectQwIsRepeat(item.getUserId());
//                if(isRepeat != null) {
//                    item.setIsRepeat(isRepeat);
//                }
            }
        }

        TableDataInfo rspData = new TableDataInfo();
        rspData.setCode(HttpStatus.SUCCESS);
        rspData.setMsg("查询成功");
        rspData.setRows(fsUserPageListVOS);

        rspData.setTotal(this.selectFsUserCount(param));
        return rspData;
    }
    @Override
    public Long selectFsUserCount(FsUserPageListParam param) {
        return fsUserMapper.selectFsUserPageListCount(param);
    }

    @Override
    public Long selectFsUserCount(int type, Long companyId) {
        return fsUserMapper.selectFsUserCount(type,companyId);
    }

    @Override
    public Map<String, Long> getUserVipCountByCompanyUserId(Long companyUserId) {
        return fsUserMapper.getUserVipCountByCompanyUserId(companyUserId);
    }

    @Override
    public FsUser selectFsUserByUnionId(String unionId) {
        return fsUserMapper.selectFsUserByUnionId(unionId);
    }

    @Override
    public UserListPageVO getUserNumber(Long userId) {
        List<UserListCountVO> list = fsUserMapper.getUserNumber(userId);
        Map<String, Integer> map = list.stream()
                .collect(Collectors.toMap(UserListCountVO::getStatus, UserListCountVO::getNum, (v1, v2) -> v1));
        UserListPageVO pageVO = new UserListPageVO();
        Integer normalNum = map.getOrDefault("1", 0);
        Integer blackNum = map.getOrDefault("0", 0);

        // 黑名单人数加上重粉的数量，正常人数去掉重粉数量
        int repeatUserNumber = fsUserMapper.getRepeatUserNumber(userId);
        pageVO.setNumber(normalNum - repeatUserNumber);
        pageVO.setBlackNum(blackNum + repeatUserNumber);

        return pageVO;
    }

    @Override
    public UserDetailsVO getUserDetails(Long userId, Long fsUserId, String dateTag) {
        UserDetailsVO countWatchCourse = fsUserMapper.getCountWatchCourse(userId, fsUserId, dateTag);
        UserDetailsVO countAnswer = fsUserMapper.getCountAnswer(userId, fsUserId, dateTag);
        UserDetailsVO countRedPacket = fsUserMapper.getCountRedPacket(userId, fsUserId, dateTag);
        UserDetailsVO vo = new UserDetailsVO();
        if (countWatchCourse != null){
            BeanUtils.copyProperties(countWatchCourse, vo);
        }
        if (countRedPacket != null) {
            vo.setAnswerRedPacketTime(countRedPacket.getAnswerRedPacketTime());
            vo.setAnswerRedPacketAmount(countRedPacket.getAnswerRedPacketAmount());
        } else {
            vo.setAnswerRedPacketAmount(BigDecimal.ZERO);
        }
        if (countAnswer != null) {
            vo.setAnswerTime(countAnswer.getAnswerTime());
            vo.setAnswerRightTime(countAnswer.getAnswerRightTime());
        }
        return vo;
    }

    @Override
    public Integer selectFsUserByUserIds(String[] userIds, Long companyUserId) {
        return fsUserMapper.selectFsUserByUserIds(userIds, companyUserId);
    }

    @Override
    public FsUserSummaryCountVO userSummaryCount(Long userId) {
        FsUserSummaryCountVO fsUserSummaryCountVO = fsUserMapper.countUserSummary(userId);
        List<FsUserSummaryCountTagVO> countTagList = fsUserMapper.countTag(userId);
        fsUserSummaryCountVO.setTagList(countTagList);
        return fsUserSummaryCountVO;
    }

    @Override
    public FsUserStatisticsVO userStatistics(UserStatisticsCommonParam param) {
        return getUserStatistics(param);
    }

    @Override
    public FsUserStatisticsVO userStatisticsDetails(UserStatisticsCommonParam param) {
        FsUserStatisticsVO userStatisticsVO = getUserStatistics(param);

        //统计课程数据详情，在查询统计详情的时候需要显示
        Map<String, Long> courseDetailsMap = fsUserMapper.countCourseDetails(param);
        if(courseDetailsMap != null && courseDetailsMap.get("courseNum") != null && courseDetailsMap.get("videoNum") != null && courseDetailsMap.get("courseUserNum") != null){
            userStatisticsVO.setCourseNum(Integer.parseInt(courseDetailsMap.get("courseNum").toString()))
                    .setVideoNum(Integer.parseInt(courseDetailsMap.get("videoNum").toString()))
                    .setCourseUserNum(Integer.parseInt(courseDetailsMap.get("courseUserNum").toString()));
        }
        return userStatisticsVO;
    }

    @Override
    public List<FsUserRankingVO> userRanking(Long userId, String startTime, String endTime, String periodId, String videoId, String order, Integer type) {
        List<FsUserRankingVO> listVO = Collections.emptyList();
        if(type == 1){
            //按完播率
            listVO = fsUserMapper.countUserRankingByComplete(userId, startTime, endTime, periodId, videoId, order);
        }
        if(type == 2){
            //按正确率
            listVO = fsUserMapper.countUserRankingByRight(userId, startTime, endTime, periodId, videoId, order);
        }
        return listVO;
    }

    @Override
    public List<FsCourseRankingVO> courseRanking(Long userId, String startTime, String endTime, String periodId, String videoId, String order, Integer type) {
        List<FsCourseRankingVO> list = Collections.emptyList();
        if(type == 1){
            list = fsUserMapper.countCourseRankingByComplete(userId, startTime, endTime, periodId, videoId, order);
        }
        if(type == 2){
            list = fsUserMapper.countCourseRankingByRight(userId, startTime, endTime, periodId, videoId, order);
        }

        return list;
    }

    @Override
    public List<FsUserGraphicStatisticsVO> graphicStatistics(UserStatisticsCommonParam param) {
        FsUserStatisticsVO userStatistics = getUserStatistics(param);
        List<FsUserGraphicStatisticsVO> list = new ArrayList<>();
        list.add(new FsUserGraphicStatisticsVO("观看人数",userStatistics.getCourseWatchNum(), userStatistics.getCourseWatchNum()));
        list.add(new FsUserGraphicStatisticsVO("完播人数", userStatistics.getCourseCompleteNum(), userStatistics.getCourseCompleteNum()));
        list.add(new FsUserGraphicStatisticsVO("答题人数", userStatistics.getAnswerNum(), userStatistics.getAnswerNum()));
        list.add(new FsUserGraphicStatisticsVO("正确人数", userStatistics.getAnswerRightNum(), userStatistics.getAnswerRightNum()));
        list.add(new FsUserGraphicStatisticsVO("答题红包数", userStatistics.getRedPacketNum(), userStatistics.getRedPacketNum()));
        list.sort(Comparator.comparingInt(FsUserGraphicStatisticsVO::getValue).reversed());
        return list;
    }

    @Override
    public List<FsCourseAnalysisVO> courseAnalysis(CourseAnalysisParam param) {
        //1、查询课程视频信息
        UserCourseVideoPageParam userCourseVideoPageParam = new UserCourseVideoPageParam();
        BeanUtils.copyProperties(param, userCourseVideoPageParam);
        List<FsUserCourseVideoPageListVO> videoList = userCourseVideoMapper.selectFsUserCourseVideoPageList(userCourseVideoPageParam);

        //2、查询统计
        List<FsCourseAnalysisCountVO> courseCountList = fsUserMapper.courseAnalysisCourseCount(param);

        List<FsCourseAnalysisCountVO> redPacketCountList = fsUserMapper.courseAnalysisRedPacketCount(param);
        List<FsCourseAnalysisCountVO> answerCountList = fsUserMapper.courseAnalysisAnswerCount(param);

        //3、转化为map
        Map<Long, FsCourseAnalysisCountVO> courseMap = courseCountList.stream()
                .collect(Collectors.toMap(
                        FsCourseAnalysisCountVO::getVideoId,
                        Function.identity()
                ));
        Map<Long, FsCourseAnalysisCountVO> redPacketMap = redPacketCountList.stream()
                .collect(Collectors.toMap(
                        FsCourseAnalysisCountVO::getVideoId,
                        Function.identity()
                ));
        Map<Long, FsCourseAnalysisCountVO> answerMap = answerCountList.stream()
                .collect(Collectors.toMap(
                        FsCourseAnalysisCountVO::getVideoId,
                        Function.identity()
                ));

        //4、处理数据
        return videoList.stream().map(v -> {
            FsCourseAnalysisVO allVO = new FsCourseAnalysisVO();
            BeanUtils.copyProperties(v, allVO);

            FsCourseAnalysisCountVO countVO = new FsCourseAnalysisCountVO();
            FsCourseAnalysisCountVO courseVO = courseMap.getOrDefault(v.getVideoId(), countVO);
            FsCourseAnalysisCountVO redPacketVO = redPacketMap.getOrDefault(v.getVideoId(), countVO);
            FsCourseAnalysisCountVO answerVO = answerMap.getOrDefault(v.getVideoId(), countVO);
            //单独赋值
            countVO.setVideoId(v.getVideoId()).setCourseWatchNum(courseVO.getCourseWatchNum()).setCourseCompleteNum(courseVO.getCourseWatchNum())
                    .setCompleteRate(courseVO.getCompleteRate() != null ? courseVO.getCompleteRate() : new BigDecimal(BigInteger.ZERO));
            countVO.setRedPacketNum(redPacketVO.getRedPacketNum())
                    .setRedPacketAmount(redPacketVO.getRedPacketAmount() != null ? redPacketVO.getRedPacketAmount() : new BigDecimal(BigInteger.ZERO));
            countVO.setAnswerNum(answerVO.getAnswerNum()).setAnswerRightNum(answerVO.getAnswerRightNum())
                    .setAnswerRightRate(answerVO.getAnswerRightRate()!=null ? answerVO.getAnswerRightRate() : new BigDecimal(BigInteger.ZERO));
            allVO.setCountVO(countVO);
            return allVO;
        }).collect(Collectors.toList());
    }

    @Override
    public CompanyUserSummaryCountVO companyUserSummaryCount(Long userId, String companyUserId) {
        CompanyUserSummaryCountVO companyUserCount = fsUserMapper.companyUserCount(companyUserId);
        CompanyUserSummaryCountVO newUserRedPacketCount = fsUserMapper.newUserRedPacketCount(companyUserId);
        CompanyUserSummaryCountVO vo = new CompanyUserSummaryCountVO();
        BeanUtils.copyProperties(companyUserCount, vo);
        vo.setUserRedPacketNum(newUserRedPacketCount.getUserRedPacketNum());
        vo.setTodayUserRedPacketAmount(newUserRedPacketCount.getTodayUserRedPacketAmount());
        return vo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResponseResult<Boolean> becomeMember(FsUserCourseBeMemberParam param) {
        //查询用户
        FsUser fsUser = fsUserMapper.selectFsUserById(param.getUserId());
        if (Objects.isNull(fsUser)){
            return ResponseResult.fail(404,"当前用户信息不存在");
        }

        // 逻辑调整：如果会员已经绑定了销售，直接提示，不添加重粉数据了-2025年6月16日14点53分
        if (fsUser.getCompanyUserId() != null && !param.getCompanyUserId().equals(fsUser.getCompanyUserId())){
            return ResponseResult.fail(406,"该用户已成为其他销售会员");
        }

        //判断该销售是否存在
        CompanyUser companyUser = companyUserMapper.selectCompanyUserById(param.getCompanyUserId());
        if (Objects.isNull(companyUser)){
            return ResponseResult.fail(405,"销售不存在");
        }

        // 添加关系表数据
        FsUserCompanyUser fsUserCompanyUser = getFsUserCompanyUser(param, fsUser);
        QueryWrapper<FsUserCompanyUser> queryWrapper = new QueryWrapper<FsUserCompanyUser>().eq("user_id", param.getUserId()).eq("company_user_id", param.getCompanyUserId());
        Integer i = fsUserCompanyUserMapper.selectCount(queryWrapper);
        if(i == 0) {
            fsUserCompanyUserMapper.insertFsUserCompanyUser(fsUserCompanyUser);
        }

        // 关联销售
        if(fsUser.getCompanyUserId() == null) {
            fsUser.setCompanyId(param.getCompanyId());
            fsUser.setCompanyUserId(param.getCompanyUserId());
        }
        // 特殊（需求设计：需要根据公司是否开启黑名单来设置会员初始化的状态）
        Company company = null;
        if(param.getCompanyId() != null) {
            company = companyMapper.selectCompanyById(param.getCompanyId());
        }
        // isDefaultBlack 值为1 ，表示需要加入小黑屋，否则不加
        int isDefaultBlack = company != null ? company.getFsUserIsDefaultBlack() : 0;
        fsUser.setStatus(isDefaultBlack == 1 ? 0 : 1);
        fsUserMapper.updateFsUser(fsUser);

        // 不为空则添加新标签
        if (Objects.nonNull(param.getTagIds()) && param.getTagIds().length > 0) {
            //关联会员标签，先删除再新增
            Map<String, Object> map = new HashMap<>();
            map.put("userId", param.getUserId());
            map.put("companyId", fsUser.getCompanyId());
            map.put("companyUserId", fsUser.getCompanyUserId());
            companyTagUserMapper.deleteCompanyTagUserByMap(map);

            CompanyTagUser companyTagUser = new CompanyTagUser();
            companyTagUser.setUserId(param.getUserId());
            companyTagUser.setCompanyId(fsUser.getCompanyId());
            companyTagUser.setCompanyUserId(fsUser.getCompanyUserId());
            companyTagUser.setTagIds(String.join(",", param.getTagIds()));
            companyTagUser.setCreateTime(new Date());
            companyTagUserMapper.insertCompanyTagUser(companyTagUser);
        }

//        // 如果是重粉，直接打上重粉的标签
//        if(1 == fsUserCompanyUser.getIsRepeatFans()){
//            this.setRepeatFansTag(param);
//        }

        //如果是设置了需要进入小黑屋，则需要返回提示，否则正常返回
        if(isDefaultBlack == 1){
            return ResponseResult.fail(402, "已成功注册，待管理审核");
        }
        return ResponseResult.ok(Boolean.TRUE);
    }


    private FsUserStatisticsVO getUserStatistics(UserStatisticsCommonParam param) {
        FsUserStatisticsVO fsUserStatisticsVO = new FsUserStatisticsVO();

        // 获取课程统计
        Map<String, Long> couserMap = fsUserMapper.countUserCourse(param);
        if (couserMap != null) {
            fsUserStatisticsVO.setCourseWatchNum(couserMap.get("courseWatchNum").intValue()).setCourseCompleteNum(couserMap.get("courseCompleteNum").intValue());

            if (couserMap.get("courseCompleteNum") != null && couserMap.get("courseWatchNum") > 0) {
                int courseCompleteRate = BigDecimal.valueOf(couserMap.get("courseCompleteNum"))
                        .divide(BigDecimal.valueOf(couserMap.get("courseWatchNum")), 2, RoundingMode.HALF_UP)
                        .multiply(BigDecimal.valueOf(100))
                        .intValue();
                fsUserStatisticsVO.setCourseCompleteRate(courseCompleteRate);
            }
        }

        // 获取答题统计
        Map<String, Long> answerMap = fsUserMapper.countUserAnswer(param);
        if (answerMap != null) {
            fsUserStatisticsVO.setAnswerNum(answerMap.get("answerNum").intValue()).setAnswerRightNum(answerMap.get("answerRightNum").intValue());

            if (answerMap.get("answerRightNum") != null && answerMap.get("answerNum") > 0) {
                int answerCompleteRate = BigDecimal.valueOf(answerMap.get("answerRightNum"))
                        .divide(BigDecimal.valueOf(answerMap.get("answerNum")), 2, RoundingMode.HALF_UP)
                        .multiply(BigDecimal.valueOf(100))
                        .intValue();
                fsUserStatisticsVO.setAnswerRightRate(answerCompleteRate);
            }
        }

        // 获取红包统计
        Map<String, Object> redPacketMap = fsUserMapper.countUserRedPacket(param);
        if(redPacketMap != null && redPacketMap.get("redPacketNum") != null && redPacketMap.get("redPacketAmount") != null) {
            fsUserStatisticsVO.setRedPacketNum(Integer.parseInt(redPacketMap.get("redPacketNum").toString()))
                    .setRedPacketAmount(new BigDecimal(redPacketMap.get("redPacketAmount").toString()));
        }
        return fsUserStatisticsVO;
    }
    private static FsUserCompanyUser getFsUserCompanyUser(FsUserCourseBeMemberParam param, FsUser fsUser) {
        FsUserCompanyUser fsUserCompanyUser = new FsUserCompanyUser();
        // 判断是否绑定了销售，如果已绑定，则需要标识为重粉，且放黑名单
        if (fsUser.getCompanyUserId() != null && !fsUser.getCompanyUserId().equals(param.getCompanyUserId())) {
            fsUserCompanyUser.setIsRepeatFans(1);
        } else {
            fsUserCompanyUser.setIsRepeatFans(0);
        }
        fsUserCompanyUser.setUserId(param.getUserId());
        fsUserCompanyUser.setCompanyId(param.getCompanyId());
        fsUserCompanyUser.setCompanyUserId(param.getCompanyUserId());
        return fsUserCompanyUser;
    }

}
