package com.fs.statis.service; // 假设Service接口放在此包下

import com.fs.statis.domain.FsStatisPeriodWatch;
import com.fs.statis.dto.StatsWatchLogPageListDTO;

import java.util.List;

/**
 * 训练营周期统计数据服务接口
 * 对应表 fs_statis_period_watch
 */
public interface FsStatisPeriodWatchService {

    /**
     * 根据主键ID查询训练营周期统计数据
     *
     * @param id 主键ID
     * @return 训练营周期统计数据对象，如果不存在则返回null
     */
    FsStatisPeriodWatch findById(Integer id);

    /**
     * 新增训练营周期统计数据
     *
     * @param fsStatisPeriodWatch 待插入的训练营周期统计数据对象
     * @return 影响的行数，通常是1表示成功
     */
    int create(FsStatisPeriodWatch fsStatisPeriodWatch);

    /**
     * 根据主键更新训练营周期统计数据
     * (只会更新实体中非null的字段)
     *
     * @param fsStatisPeriodWatch 待更新的训练营周期统计数据对象 (必须包含ID)
     * @return 影响的行数，通常是1表示成功，0表示未找到对应记录或未更新任何字段
     */
    int updateById(FsStatisPeriodWatch fsStatisPeriodWatch);

    /**
     * 根据主键ID删除训练营周期统计数据
     *
     * @param id 主键ID
     * @return 影响的行数
     */
    int deleteById(Integer id);

    /**
     * 查询所有训练营周期统计数据
     *
     * @return 训练营周期统计数据列表
     */
    List<FsStatisPeriodWatch> findAll();

    List<FsStatisPeriodWatch> queryList(StatsWatchLogPageListDTO param);

}
